@@ -8,18 +8,22 @@
 // <int>.<int>.<int>-<any text>.
 // or else upgrading old project file will not work
 // reliably.
-#define VERSION "3.6.1-Noosa"
+// TODO QGIS 4: remove in favor of _QGIS_VERSION
+#define VERSION "3.10.1-A Coruña"
+#define _QGIS_VERSION "3.10.1-A Coruña"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 361
-#define VERSION_INT 30601
-#define ABISYM(x) x ## 30601
+//#define VERSION_INT 3101
+// TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
+#define VERSION_INT 31001
+#define _QGIS_VERSION_INT 31001
+#define ABISYM(x) x ## 31001
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Noosa"
+#define RELEASE_NAME "A Coruña"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -27,11 +31,11 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.6.1"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.10.1"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
-#define QSCINTILLA_VERSION_STR "2.11.1"
+#define QSCINTILLA_VERSION_STR "2.11.3"
 
 #if defined( __APPLE__ )
 //used by Mac to find system or bundle resources relative to amount of bundling
@@ -71,5 +75,7 @@
 
 #define HAVE_QT5SERIALPORT
 
+/* #undef HAVE_STATIC_PROVIDERS */
+
 #endif
 
