@@ -36,6 +36,7 @@
 #include "qgssnappingconfig.h"
 #include "qgsprojectversion.h"
 #include "qgsexpressioncontextgenerator.h"
+#include "qgsexpressioncontextscopegenerator.h"
 #include "qgscoordinatereferencesystem.h"
 #include "qgscoordinatetransformcontext.h"
 #include "qgsprojectproperty.h"
@@ -70,23 +71,23 @@ class QgsLayerTree;
 class QgsLabelingEngineSettings;
 class QgsAuxiliaryStorage;
 class QgsMapLayer;
+class QgsBookmarkManager;
+class QgsProjectViewSettings;
 
 /**
  * \ingroup core
- * Reads and writes project states.
+ * Encapsulates a QGIS project, including sets of map layers and their styles,
+ * layouts, annotations, canvases, etc.
  *
-  \note
-
-  Has two general kinds of state to make persistent.  (I.e., to read and
-  write.)  First, QGIS proprietary information.  Second plug-in information.
-
-  A singleton since there shall only be one active project at a time; and
-  provides canonical location for plug-ins and main app to find/set
-  properties.
-
+ * QgsProject is available both as a singleton (QgsProject::instance()) and for use as
+ * standalone objects. The QGIS project singleton always gives access to the canonical project reference
+ * open within the main QGIS application.
+ *
+ * \note QgsProject has two general kinds of state to make persistent. (I.e., to read and
+ * write.) First, QGIS proprietary information. Second plugin information.
 */
 
-class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenerator, public QgsProjectTranslator
+class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenerator, public QgsExpressionContextScopeGenerator, public QgsProjectTranslator
 {
     Q_OBJECT
     Q_PROPERTY( QStringList nonIdentifiableLayers READ nonIdentifiableLayers WRITE setNonIdentifiableLayers NOTIFY nonIdentifiableLayersChanged )
@@ -100,6 +101,8 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     Q_PROPERTY( QgsRelationManager *relationManager READ relationManager )
     Q_PROPERTY( QList<QgsVectorLayer *> avoidIntersectionsLayers READ avoidIntersectionsLayers WRITE setAvoidIntersectionsLayers NOTIFY avoidIntersectionsLayersChanged )
     Q_PROPERTY( QgsProjectMetadata metadata READ metadata WRITE setMetadata NOTIFY metadataChanged )
+    Q_PROPERTY( QColor backgroundColor READ backgroundColor WRITE setBackgroundColor NOTIFY backgroundColorChanged )
+    Q_PROPERTY( QColor selectionColor READ selectionColor WRITE setSelectionColor NOTIFY selectionColorChanged )
 
   public:
     //! Returns the QgsProject singleton instance
@@ -134,12 +137,12 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     QString title() const;
 
     /**
-     * Returns true if the project has been modified since the last write()
+     * Returns TRUE if the project has been modified since the last write()
      */
     bool isDirty() const;
 
     /**
-     * Sets the file name associated with the project. This is the file which contains the project's XML
+     * Sets the file name associated with the project. This is the file or the storage URI which contains the project's XML
      * representation.
      * \param name project file name
      * \see fileName()
@@ -147,7 +150,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     void setFileName( const QString &name );
 
     /**
-     * Returns the project's file name. This is the file which contains the project's XML
+     * Returns the project's file name. This is the file or the storage URI which contains the project's XML
      * representation.
      * \see setFileName()
      * \see fileInfo()
@@ -162,14 +165,13 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * replacements that are aware of the fact that projects may be saved in other project storages.
      *
      * \see fileName()
-     * \since QGIS 2.9
-     * \deprecated Use absoluteFilePath(), baseName() or lastModifiedTime() instead
+     * \deprecated since QGIS 3.2 use absoluteFilePath(), baseName() or lastModifiedTime() instead
      */
     Q_DECL_DEPRECATED QFileInfo fileInfo() const SIP_DEPRECATED;
 
     /**
      * Returns pointer to project storage implementation that handles read/write of the project file.
-     * If the project file is stored in the local file system, returns null pointer.
+     * If the project file is stored in the local file system, returns NULLPTR.
      * The project storage object is inferred from fileName() of the project.
      * \since QGIS 3.2
      */
@@ -211,11 +213,14 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
 
     /**
      * Sets the project's native coordinate reference system.
+     * If \a adjustEllipsoid is set to TRUE, the ellpsoid of this project will be set to
+     * the ellipsoid imposed by the CRS.
+     *
      * \see crs()
      * \see setEllipsoid()
      * \since QGIS 3.0
      */
-    void setCrs( const QgsCoordinateReferenceSystem &crs );
+    void setCrs( const QgsCoordinateReferenceSystem &crs, bool adjustEllipsoid = false );
 
     /**
      * Returns a proj string representing the project's ellipsoid setting, e.g., "WGS84".
@@ -263,18 +268,34 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     void clear();
 
+    /**
+     * Flags which control project read behavior.
+     * \since QGIS 3.10
+     */
+    enum ReadFlag
+    {
+      FlagDontResolveLayers = 1 << 0, //!< Don't resolve layer paths (i.e. don't load any layer content). Dramatically improves project read time if the actual data from the layers is not required.
+      FlagDontLoadLayouts = 1 << 1, //!< Don't load print layouts. Improves project read time if layouts are not required, and allows projects to be safely read in background threads (since print layouts are not thread safe).
+    };
+    Q_DECLARE_FLAGS( ReadFlags, ReadFlag )
+
     /**
      * Reads given project file from the given file.
      * \param filename name of project file to read
-     * \returns true if project file has been read successfully
+     * \param flags optional flags which control the read behavior of projects
+     * \returns TRUE if project file has been read successfully
      */
-    bool read( const QString &filename );
+    bool read( const QString &filename, QgsProject::ReadFlags flags = nullptr );
 
     /**
      * Reads the project from its currently associated file (see fileName() ).
-     * \returns true if project file has been read successfully
+     *
+     * The \a flags argument can be used to specify optional flags which control
+     * the read behavior of projects.
+     *
+     * \returns TRUE if project file has been read successfully
      */
-    bool read();
+    bool read( QgsProject::ReadFlags flags = nullptr );
 
     /**
      * Reads the layer described in the associated DOM node.
@@ -290,17 +311,17 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     /**
      * Writes the project to a file.
      * \param filename destination file
-     * \returns true if project was written successfully
+     * \returns TRUE if project was written successfully
      * \note calling this implicitly sets the project's filename (see setFileName() )
-     * \note isDirty() will be set to false if project is successfully written
+     * \note isDirty() will be set to FALSE if project is successfully written
      * \since QGIS 3.0
      */
     bool write( const QString &filename );
 
     /**
      * Writes the project to its current associated file (see fileName() ).
-     * \returns true if project was written successfully
-     * \note isDirty() will be set to false if project is successfully written
+     * \returns TRUE if project was written successfully
+     * \note isDirty() will be set to FALSE if project is successfully written
      */
     bool write();
 
@@ -426,18 +447,24 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     QString layerIsEmbedded( const QString &id ) const;
 
     /**
-     * Creates a maplayer instance defined in an arbitrary project file. Caller takes ownership
+     * Creates a maplayer instance defined in an arbitrary project file. Caller takes ownership.
+     *
+     * The optional \a flags argument can be used to specify flags which control layer reading.
+     *
      * \returns the layer or 0 in case of error
      * \note not available in Python bindings
      */
     bool createEmbeddedLayer( const QString &layerId, const QString &projectFilePath, QList<QDomNode> &brokenNodes,
-                              bool saveFlag = true ) SIP_SKIP;
+                              bool saveFlag = true, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
 
     /**
      * Create layer group instance defined in an arbitrary project file.
+     *
+     * The optional \a flags argument can be used to control layer reading behavior.
+     *
      * \since QGIS 2.4
      */
-    QgsLayerTreeGroup *createEmbeddedGroup( const QString &groupName, const QString &projectFilePath, const QStringList &invisibleLayers );
+    QgsLayerTreeGroup *createEmbeddedGroup( const QString &groupName, const QString &projectFilePath, const QStringList &invisibleLayers, QgsProject::ReadFlags flags = nullptr );
 
     //! Convenience function to set topological editing
     void setTopologicalEditing( bool enabled );
@@ -521,6 +548,38 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     QgsLayoutManager *layoutManager();
 
+    /**
+     * Returns the project's bookmark manager, which manages bookmarks within
+     * the project.
+     * \note not available in Python bindings
+     * \since QGIS 3.10
+     */
+    const QgsBookmarkManager *bookmarkManager() const SIP_SKIP;
+
+    /**
+     * Returns the project's bookmark manager, which manages bookmarks within
+     * the project.
+     * \since QGIS 3.10
+     */
+    QgsBookmarkManager *bookmarkManager();
+
+    /**
+     * Returns the project's view settings, which contains settings and properties
+     * relating to how a QgsProject should be viewed and behave inside a map canvas
+     * (e.g. map scales and default view extent)
+     * \note not available in Python bindings
+     * \since QGIS 3.10.1
+     */
+    const QgsProjectViewSettings *viewSettings() const SIP_SKIP;
+
+    /**
+     * Returns the project's view settings, which contains settings and properties
+     * relating to how a QgsProject should be viewed and behave inside a map canvas
+     * (e.g. map scales and default view extent)
+     * \since QGIS 3.10.1
+     */
+    QgsProjectViewSettings *viewSettings();
+
     /**
      * Returns pointer to the root (invisible) node of the project's layer tree
      * \since QGIS 2.4
@@ -603,7 +662,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     /**
      * Returns the matching transaction group from a provider key and connection string.
      *
-     * Returns nullptr if a matching transaction group is not available.
+     * Returns NULLPTR if a matching transaction group is not available.
      *
      * \since QGIS 3.2
      */
@@ -624,6 +683,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     void setEvaluateDefaultValues( bool evaluateDefaultValues );
 
     QgsExpressionContext createExpressionContext() const override;
+    QgsExpressionContextScope *createExpressionContextScope() const override;
 
     /**
      * The snapping configuration for this project.
@@ -696,18 +756,18 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     /**
      * Retrieve a pointer to a registered layer by layer ID.
      * \param layerId ID of layer to retrieve
-     * \returns matching layer, or nullptr if no matching layer found
+     * \returns matching layer, or NULLPTR if no matching layer found
      * \see mapLayersByName()
      * \see mapLayers()
      */
-    QgsMapLayer *mapLayer( const QString &layerId ) const;
+    Q_INVOKABLE QgsMapLayer *mapLayer( const QString &layerId ) const;
 
 #ifndef SIP_RUN
 
     /**
      * Retrieve a pointer to a registered layer by \p layerId converted
      * to type T. This is a convenience template.
-     * A nullptr will be returned if the layer is not found or
+     * A NULLPTR will be returned if the layer is not found or
      * if it cannot be cast to type T.
      *
      * \code{cpp}
@@ -735,6 +795,18 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     QList<QgsMapLayer *> mapLayersByName( const QString &layerName ) const;
 
+    /**
+     * Retrieves a list of matching registered layers by layer \a shortName.
+     * If layer's short name is empty a match with layer's name is attempted.
+     *
+     * \returns list of matching layers
+     * \see mapLayer()
+     * \see mapLayers()
+     * \since QGIS 3.10
+     */
+    QList<QgsMapLayer *> mapLayersByShortName( const QString &shortName ) const;
+
+
     /**
      * Returns a map of all registered layers by layer ID.
      *
@@ -746,7 +818,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     QMap<QString, QgsMapLayer *> mapLayers( const bool validOnly = false ) const;
 
     /**
-     * Returns true if the project comes from a zip archive, false otherwise.
+     * Returns TRUE if the project comes from a zip archive, FALSE otherwise.
      */
     bool isZipped() const;
 
@@ -768,6 +840,38 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     {
       return mLayerStore->layers<T>();
     }
+
+    /**
+     * Retrieves a list of matching registered layers by layer \a shortName with a specified layer type,
+     * if layer's short name is empty a match with layer's name is attempted.
+     *
+     * \param shortName short name of layers to match
+     * \returns list of matching layers
+     * \see mapLayer()
+     * \see mapLayers()
+     * \note not available in Python bindings
+     * \since QGIS 3.10
+     */
+    template <typename T>
+    QVector<T> mapLayersByShortName( const QString &shortName ) const
+    {
+      QVector<T> layers;
+      const auto constMapLayers { mLayerStore->layers<T>() };
+      for ( const auto l : constMapLayers )
+      {
+        if ( ! l->shortName().isEmpty() )
+        {
+          if ( l->shortName() == shortName )
+            layers << l;
+        }
+        else if ( l->name() == shortName )
+        {
+          layers << l;
+        }
+      }
+      return layers;
+    }
+
 #endif
 
     /**
@@ -775,14 +879,14 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * Add a list of layers to the map of loaded layers.
      *
      * The layersAdded() and layerWasAdded() signals will always be emitted.
-     * The legendLayersAdded() signal is emitted only if addToLegend is true.
+     * The legendLayersAdded() signal is emitted only if addToLegend is TRUE.
      *
      * \param mapLayers  A list of layer which should be added to the registry
-     * \param addToLegend   If true (by default), the layers will be added to the
+     * \param addToLegend   If TRUE (by default), the layers will be added to the
      *                      legend and to the main canvas. If you have a private
-     *                      layer you can set this parameter to false to hide it.
+     *                      layer you can set this parameter to FALSE to hide it.
      * \param takeOwnership Ownership will be transferred to the layer registry.
-     *                      If you specify false here you have take care of deleting
+     *                      If you specify FALSE here you have take care of deleting
      *                      the layers yourself. Not available in Python.
      *
      * \returns a list of the map layers that were added
@@ -804,19 +908,19 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * Add a layer to the map of loaded layers.
      *
      * The layersAdded() and layerWasAdded() signals will always be emitted.
-     * The legendLayersAdded() signal is emitted only if addToLegend is true.
+     * The legendLayersAdded() signal is emitted only if addToLegend is TRUE.
      * If you are adding multiple layers at once, you should use
      * addMapLayers() instead.
      *
      * \param mapLayer A layer to add to the registry
-     * \param addToLegend If true (by default), the layer will be added to the
+     * \param addToLegend If TRUE (by default), the layer will be added to the
      *                    legend and to the main canvas. If you have a private
-     *                    layer you can set this parameter to false to hide it.
+     *                    layer you can set this parameter to FALSE to hide it.
      * \param takeOwnership Ownership will be transferred to the layer registry.
-     *                      If you specify false here you have take care of deleting
+     *                      If you specify FALSE here you have take care of deleting
      *                      the layer yourself. Not available in Python.
      *
-     * \returns nullptr if unable to add layer, otherwise pointer to newly added layer
+     * \returns NULLPTR if unable to add layer, otherwise pointer to newly added layer
      *
      * \see addMapLayers
      *
@@ -855,7 +959,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * The specified layers will be removed from the registry. If the registry has ownership
      * of any layers these layers will also be deleted.
      *
-     * \param layers A list of layers to remove. Null pointers are ignored.
+     * \param layers A list of layers to remove. NULLPTR values are ignored.
      *
      * \note As a side-effect the QgsProject instance is marked dirty.
      * \see removeMapLayer()
@@ -885,7 +989,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * The specified layer will be removed from the registry. If the registry has ownership
      * of the layer then it will also be deleted.
      *
-     * \param layer The layer to remove. Null pointers are ignored.
+     * \param layer The layer to remove. NULLPTR values are ignored.
      *
      * \note As a side-effect the QgsProject instance is marked dirty.
      * \see removeMapLayers()
@@ -934,14 +1038,14 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * activated, primary key unicity is not checked for views and
      * materialized views with Postgres provider.
      *
-     * \param trust True to trust the project, false otherwise
+     * \param trust TRUE to trust the project, FALSE otherwise
      *
      * \since QGIS 3.0
      */
     void setTrustLayerMetadata( bool trust );
 
     /**
-     * Returns true if the trust option is activated, false otherwise. This
+     * Returns TRUE if the trust option is activated, FALSE otherwise. This
      * option allows indicateing if the extent has to be read from the XML
      * document when data source has no metadata or if the data provider has
      * to determine it. Moreover, when this option is activated, primary key
@@ -966,6 +1070,27 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     QgsAuxiliaryStorage *auxiliaryStorage();
 
+    /**
+     * Returns the path to an attached file known by \a fileName.
+     *
+     * \note Not available in Python bindings
+     * \note Attached files are only supported by QGZ file based projects
+     * \see collectAttachedFiles()
+     * \since QGIS 3.8
+     */
+    QString attachedFile( const QString &fileName ) const SIP_SKIP;
+
+    /**
+     * Returns a map of all attached files with relative paths and real paths.
+     *
+     * \note Not available in Python bindings
+     * \note Attached files are only supported by QGZ file based projects
+     * \see collectAttachedFiles()
+     * \see attachedFile()
+     * \since QGIS 3.8
+     */
+    QgsStringMap attachedFiles() const SIP_SKIP;
+
     /**
      * Returns a reference to the project's metadata store.
      * \see setMetadata()
@@ -1010,6 +1135,84 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     void setProjectColors( const QgsNamedColorList &colors );
 
+    /**
+     * Sets the default background \a color used by default map canvases.
+     *
+     * \see backgroundColor()
+     * \since QGIS 3.10
+     */
+    void setBackgroundColor( const QColor &color );
+
+    /**
+     * Returns the default background color used by default map canvases.
+     *
+     * \see setBackgroundColor()
+     * \since QGIS 3.10
+     */
+    QColor backgroundColor() const;
+
+    /**
+     * Sets the \a color used to highlight selected features.
+     *
+     * \see selectionColor()
+     * \since QGIS 3.10
+     */
+    void setSelectionColor( const QColor &color );
+
+    /**
+     * Returns the color used to highlight selected features
+     *
+     * \see setSelectionColor()
+     * \since QGIS 3.10
+     */
+    QColor selectionColor() const;
+
+    /**
+     * Sets the list of custom project map \a scales.
+     *
+     * The \a scales list consists of a list of scale denominator values, e.g.
+     * 1000 for a 1:1000 scale.
+     *
+     * \see mapScales()
+     * \see mapScalesChanged()
+     *
+     * \deprecated Use viewSettings() instead
+     */
+    Q_DECL_DEPRECATED void setMapScales( const QVector<double> &scales ) SIP_DEPRECATED;
+
+    /**
+     * Returns the list of custom project map scales.
+     *
+     * The scales list consists of a list of scale denominator values, e.g.
+     * 1000 for a 1:1000 scale.
+     *
+     * \see setMapScales()
+     * \see mapScalesChanged()
+     *
+     * \deprecated Use viewSettings() instead
+     */
+    Q_DECL_DEPRECATED QVector<double> mapScales() const SIP_DEPRECATED;
+
+    /**
+     * Sets whether project mapScales() are \a enabled.
+     *
+     * \see useProjectScales()
+     * \see setMapScales()
+     *
+     * \deprecated Use viewSettings() instead
+     */
+    Q_DECL_DEPRECATED void setUseProjectScales( bool enabled ) SIP_DEPRECATED;
+
+    /**
+     * Returns TRUE if project mapScales() are enabled.
+     *
+     * \see setUseProjectScales()
+     * \see mapScales()
+     *
+     * \deprecated Use viewSettings() instead
+     */
+    Q_DECL_DEPRECATED bool useProjectScales() const SIP_DEPRECATED;
+
     /**
      * Triggers the collection strings of .qgs to be included in ts file and calls writeTsFile()
      * \since QGIS 3.4
@@ -1028,6 +1231,26 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     QString translate( const QString &context, const QString &sourceText, const char *disambiguation = nullptr, int n = -1 ) const override;
 
+    /**
+     * Accepts the specified style entity \a visitor, causing it to visit all style entities associated
+     * with the project.
+     *
+     * Returns TRUE if the visitor should continue visiting other objects, or FALSE if visiting
+     * should be canceled.
+     *
+     * \since QGIS 3.10
+     */
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const;
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString str = QStringLiteral( "<QgsProject: '%1'%2>" ).arg( sipCpp->fileName(),
+                  sipCpp == QgsProject::instance() ? QStringLiteral( " (singleton instance)" ) : QString() );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
   signals:
 
     /**
@@ -1218,6 +1441,22 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      */
     void projectColorsChanged();
 
+    /**
+     * Emitted whenever the project's canvas background color has been changed.
+
+     * \see setBackgroundColor()
+     * \since QGIS 3.10
+     */
+    void backgroundColorChanged();
+
+    /**
+     * Emitted whenever the project's selection color has been changed.
+
+     * \see setSelectionColor()
+     * \since QGIS 3.10
+     */
+    void selectionColorChanged();
+
     //
     // signals from QgsMapLayerRegistry
     //
@@ -1323,12 +1562,37 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     /**
      * Emitted when the project dirty status changes.
      *
-     * \param dirty True if the project is in a dirty state and has pending unsaved changes.
+     * \param dirty TRUE if the project is in a dirty state and has pending unsaved changes.
      *
      * \since QGIS 3.2
      */
     void isDirtyChanged( bool dirty );
 
+    /**
+     * Emitted whenever the project is saved to a qgz file.
+     * This can be used to package additional files into the qgz file by modifying the \a files map.
+     *
+     * Map keys represent relative paths inside the qgz file, map values represent the path to
+     * the source file.
+     *
+     * \note Not available in Python bindings
+     * \note Only will be emitted with QGZ project files
+     * \see attachedFiles()
+     * \see attachedFile()
+     * \since QGIS 3.8
+     */
+    void collectAttachedFiles( QgsStringMap &files SIP_INOUT ) SIP_SKIP;
+
+    /**
+     * Emitted when the list of custom project map scales changes.
+     *
+     * \see mapScales()
+     * \see setMapScales()
+     *
+     * \deprecated Use viewSettings() instead
+     */
+    Q_DECL_DEPRECATED void mapScalesChanged() SIP_DEPRECATED;
+
   public slots:
 
     /**
@@ -1402,9 +1666,10 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * \param doc DOM document to parse
      * \param brokenNodes a list of DOM nodes corresponding to layers that we were unable to load; this could be
      * because the layers were removed or re-located after the project was last saved
-     * \returns true if function worked; else is false
+     * \param flags optional project reading flags
+     * \returns TRUE if function worked; else is FALSE
     */
-    bool _getMapLayers( const QDomDocument &doc, QList<QDomNode> &brokenNodes );
+    bool _getMapLayers( const QDomDocument &doc, QList<QDomNode> &brokenNodes, QgsProject::ReadFlags flags = nullptr );
 
     /**
      * Set error message from read/write operation
@@ -1419,25 +1684,35 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     void clearError() SIP_SKIP;
 
     /**
-     * Creates layer and adds it to maplayer registry
+     * Creates layer and adds it to maplayer registry.
+     *
+     * The optional \a flags argument can be used to control layer reading behavior.
+     *
      * \note not available in Python bindings
      */
-    bool addLayer( const QDomElement &layerElem, QList<QDomNode> &brokenNodes, QgsReadWriteContext &context ) SIP_SKIP;
+    bool addLayer( const QDomElement &layerElem, QList<QDomNode> &brokenNodes, QgsReadWriteContext &context, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
 
-    //! \note not available in Python bindings
-    void initializeEmbeddedSubtree( const QString &projectFilePath, QgsLayerTreeGroup *group ) SIP_SKIP;
+    /**
+     * The optional \a flags argument can be used to control layer reading behavior.
+     *
+     * \note not available in Python bindings
+    */
+    void initializeEmbeddedSubtree( const QString &projectFilePath, QgsLayerTreeGroup *group, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
 
-    //! \note not available in Python bindings
-    void loadEmbeddedNodes( QgsLayerTreeGroup *group ) SIP_SKIP;
+    /**
+     * The optional \a flags argument can be used to control layer reading behavior.
+     * \note not available in Python bindings
+     */
+    void loadEmbeddedNodes( QgsLayerTreeGroup *group, QgsProject::ReadFlags flags = nullptr ) SIP_SKIP;
 
     //! Read .qgs file
-    bool readProjectFile( const QString &filename );
+    bool readProjectFile( const QString &filename, QgsProject::ReadFlags flags = nullptr );
 
     //! Write .qgs file
     bool writeProjectFile( const QString &filename );
 
     //! Unzip .qgz file then read embedded .qgs file
-    bool unzip( const QString &filename );
+    bool unzip( const QString &filename, QgsProject::ReadFlags flags = nullptr );
 
     //! Zip project
     bool zip( const QString &filename );
@@ -1465,6 +1740,10 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     std::unique_ptr<QgsAnnotationManager> mAnnotationManager;
     std::unique_ptr<QgsLayoutManager> mLayoutManager;
 
+    QgsBookmarkManager *mBookmarkManager = nullptr;
+
+    QgsProjectViewSettings *mViewSettings = nullptr;
+
     QgsLayerTree *mRootGroup = nullptr;
 
     QgsLayerTreeRegistryBridge *mLayerTreeRegistryBridge = nullptr;
@@ -1489,6 +1768,11 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      * created based on file name.
      */
     QString mHomePath;
+    mutable QString mCachedHomePath;
+
+    QColor mBackgroundColor;
+    QColor mSelectionColor;
+
     mutable QgsProjectPropertyKey mProperties;  // property hierarchy, TODO: this shouldn't be mutable
     bool mAutoTransaction = false;       // transaction grouped editing
     bool mEvaluateDefaultValues = false; // evaluate default values immediately
@@ -1505,6 +1789,8 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
 
     bool mIsBeingDeleted = false;
 
+    mutable std::unique_ptr< QgsExpressionContextScope > mProjectScope;
+
     friend class QgsProjectDirtyBlocker;
 
     // Required to avoid creating a new project in it's destructor
@@ -1514,6 +1800,8 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
     friend class QgsConfigCache;
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProject::ReadFlags )
+
 /**
  * Temporarily blocks QgsProject "dirtying" for the lifetime of the object.
  *
@@ -1576,4 +1864,31 @@ class CORE_EXPORT QgsProjectDirtyBlocker
  */
 CORE_EXPORT QgsProjectVersion getVersion( QDomDocument const &doc ) SIP_SKIP;
 
+
+
+/// @cond PRIVATE
+#ifndef SIP_RUN
+class GetNamedProjectColor : public QgsScopedExpressionFunction
+{
+  public:
+    GetNamedProjectColor( const QgsProject *project );
+
+    /**
+     * Optimized constructor for GetNamedProjectColor when a list of map is already available
+     * and does not need to be read from a project.
+     */
+    GetNamedProjectColor( const QHash< QString, QColor > &colors );
+
+    QVariant func( const QVariantList &values, const QgsExpressionContext *, QgsExpression *, const QgsExpressionNodeFunction * ) override;
+    QgsScopedExpressionFunction *clone() const override;
+
+  private:
+
+    QHash< QString, QColor > mColors;
+
+};
+
+#endif
+///@endcond
+
 #endif
