@@ -21,10 +21,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
-
 import os
 from qgis.core import (QgsProcessingParameterRasterLayer,
                        QgsProcessingParameterRasterDestination)
@@ -60,7 +56,10 @@ class SplitRGBBands(SagaAlgorithmBase):
         return self.tr('Split RGB bands')
 
     def group(self):
-        return self.tr('Image tools')
+        return self.tr('Raster tools')
+
+    def groupId(self):
+        return 'rastertools'
 
     def processAlgorithm(self, parameters, context, feedback):
         # TODO: check correct num of bands
